#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

#---------------------------------------------------------------------#
# Loads an image (using the image_loader routine)                     #
#---------------------------------------------------------------------#
proc Load_image {{filename ""}} {

    global imagefile
    
    set imagefile $filename
    if {$imagefile == ""} {
        set msg "Enter name of image file (*.xbm or *.rppm). \
            (In future releases this will be replaced by a file browser)." 
        set imagefile [getEntry $msg ]
    }
    set ext [file extension $imagefile]
    if {$ext == ".xbm" || $ext == ".rppm" || $ext == ".gif"} {
        Image_loader $imagefile
    } else {
        postMessage "Cannot read Image file with extension: $ext"
    }
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

#---------------------------------------------------------------------#
# general purpose box that asks for a string (default value possible) #
#---------------------------------------------------------------------#
proc getEntry {msg {default ""}} {
    global entryval

# destroy window if it exists
    if {[winfo exists .entry]} {
        destroy .entry
    }
# make a toplevel window
    toplevel .entry 
    message .entry.msg -text $msg -width 500
    pack .entry.msg
    entry .entry.entry -relief sunken 
    pack .entry.entry -side top -expand yes -fill x
    frame .entry.bottom 
    pack .entry.bottom -expand yes -fill x
    button .entry.bottom.ok -text OK \
        -command {set entryval [.entry.entry get]}
    button .entry.bottom.cancel -text CANCEL \
        -command {set entryval ""} 
    pack .entry.bottom.ok -side left -padx 120
    pack .entry.bottom.cancel -side right -padx 120
    bind .entry.entry <Return> {.entry.bottom.ok invoke}
    .entry.entry insert 0 $default
    tkwait variable entryval
    destroy .entry
    return $entryval
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

# Create a toplevel filebrowser letting user pick a mapfile to read from
proc Read_mapfile {} {

    global mapfile

    set msg "Enter name of existing map file (NCSA format only at present)"
    set mapfile [getEntry $msg $mapfile]
    if {$mapfile == ""} {
        return
    }

    set mapfile [addFileExtension $mapfile map]

    Read_Map_Info $mapfile
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

#---------------------------------------------------------------------#
#---------------------------------------------------------------------#
proc Read_Map_Info {file_with_map} {

    global mapfile defaulthtml imagefile
    global _NITEM
    set _NITEM 0

    if {![file exists $file_with_map]} {
	postMessage "The selected file does not exist.  Please select again."
        return
    }
# set some defaults
    set Colour red
    set LineWidth 4
    set Type ""

# Store the contents of the file containing the map in a list.  
# Each line will be held as a list element
    set Contents [GetFileContents $file_with_map]

# we are assuming that each file starts as:
#    #comment [// mapfilename]
#    default <url>

    set defhtml 0
    set Index 0
    while {$Index < [llength $Contents]} {
        set line [lindex $Contents $Index]
        incr Index
# whitespace
        if {[regexp {^ *$} $line]} {
            continue
        } elseif {[string range $line 0 0] == "#"} {
# comments
            processComment $line Colour LineWidth imagefile Comments
        } else {
            set head [string toupper [string range $line 0 3]]
            if {$head == "RECT" ||
                $head == "CIRC" ||
                $head == "POLY"} { 
                set Type [extractURLCoords $line URL Coords]
                drawMapFileObject $Coords $Type $Colour $LineWidth \
                    $Comments $URL
                incr _NITEM
            } elseif {$head == "DEFA"} {
                set defaulthtml [string trim [string range \
                       [lindex $Contents $Index] 7 end]]
                set defhtml 1
            } else {
                postMessage "uninterpretable/misplaced line in map file: $line"
            }
        }
    }
    if {$defhtml != 1} {
        postMessage "Warning: Map file has no default HTML"
    }
# Invoke the menu-button which sets the default bindings.
# This allows any loaded shapes to be selected.
    .can.menu.create.men invoke @0
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

#---------------------------------------------------------------------#
proc processComment {line Col LineW ImageF Comms} {

    upvar 1 $Col Colour
    upvar 1 $LineW LineWidth
    upvar 1 $ImageF ImageFile
    upvar 1 $Comms Comments

# behead the comment
    set line [string trim [string range $line 1 end]]
    set words [split $line \ ]
    set nwords 0
    while {$nwords < [llength $words]} {
        set first [string range [string tolower [lindex $words $nwords]] 0 4]
        incr nwords
        set second [lindex $words $nwords]
        incr nwords
        if {$first == "_imag"} {
            set ImageFile $second
            if {$ImageFile != ""} {
                if {[yesNo "Load the image file $ImageFile"] == "YES"} {
                    Load_image $ImageFile
                }
            }
        } elseif {$first == "_colo"} {
            set Colour $second
        } elseif {$first == "_line"} {
            set LineWidth $second
        } elseif {$first == ""} {
            break
        } else {
            set Comments $line
        }
    }
}
    
#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

#---------------------------------------------------------------------#
proc extractURLCoords {Line u c} {

    upvar 1 $u URL
    upvar 1 $c Coords 

# get the map line
    regsub -all "  " $Line " " Line
    set words [split $Line \ ]
    if {[llength $words] < 3} {
        postMessage "Bad (Map) line in file at: $Line"
        return
    }
    
    set temp [string tolower [string range [lindex $words 0] 0 3]]
    if {$temp == "rect"} {
        set Type rectangle
    }
    if {$temp == "circ"} {
        set Type oval
    }
    if {$temp == "poly"} {
        set Type polygon
    }
    
    set URL [lindex $words 1]
    
# extract the coords
    set End [llength $Line]
    set Coords0 [lrange $Line 2 $End]
    regsub -all "," $Coords0 " " Coords

    return $Type
    
}
#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

proc drawMapFileObject {Coords Type Colour LineWidth Comments URL} {

    global _NITEM
    set IndexTag item$_NITEM

    if {$Type=="oval"} {
        set radius [expr ([lindex $Coords 3]-[lindex $Coords 1])]
        set x1 [expr ([lindex $Coords 0]-$radius)]
        set x2 [expr ([lindex $Coords 0]+$radius)]
        set y1 [expr ([lindex $Coords 1]-$radius)]
        set y2 [expr ([lindex $Coords 1]+$radius)]
        set Coords "$x1 $y1 $x2 $y2"
    }
    
    if {$LineWidth!=0} {
        if {$Type=="polygon"} {
            eval ".can.c1 create line $Coords -fill $Colour -width $LineWidth \
                -tags $IndexTag"
        } else {
            eval ".can.c1 create $Type $Coords -outline $Colour -fill {} \
                -width $LineWidth -tags $IndexTag"
        }
    } else {
        if {$Type=="polygon"} {
            eval ".can.c1 create polygon $Coords -fill $Colour  \
                 -tags $IndexTag"
        } else {
            eval ".can.c1 create $Type $Coords -outline {} \
                -fill $Colour -tags $IndexTag"
        }
    }
    set Attr "$Type $Colour $LineWidth  \{$Comments\} $URL"
    .can.c1 addtag $Attr withtag $IndexTag
    .can.c1 addtag item withtag $IndexTag
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

proc GetFileContents {filename} {


    set f [open $filename r]
    if {$f == ""} {
        return ""
    }
    set list ""
    while {1} {
        if {[eof $f]} {
            break
        }
        gets $f line
        lappend list $line
    }
    return $list
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#


#---------------------------------------------------------------------#
# general purpose box that posts a message                            #
#---------------------------------------------------------------------#
proc postMessage {msg} {
    global messageflag

    toplevel .message
    message .message.msg -text $msg -width 500
    pack .message.msg
    frame .message.bottom 
    pack .message.bottom -expand yes -fill x
    button .message.bottom.ok -text OK \
        -command {set messageflag 1; destroy .message}
    pack .message.bottom.ok
    tkwait variable messageflag
}


#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

#---------------------------------------------------------------------#
# adds an extension to the filename if one isn't given                #
#---------------------------------------------------------------------#
proc addFileExtension {filename ext} {

    if {$filename == ""} {
        return ""
    }

# set standard extension
    set extension [file extension $filename]
    if {$extension == ""} {
        set filename $filename.$ext
    }
    return $filename
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

#---------------------------------------------------------------------#
# saves the map to the current map file                               #
#---------------------------------------------------------------------#
proc Save_Map_To_File {{mfile ""}} {

    global defaulthtml mapfile imagefile
    global _NITEM

# check for unannotated objects
    set object [SelectFirstUnannotated]
    if {$object != -1} {
        Auto_Select item$object
        postMessage "There are unannotated objects; the first has been selected"
        Annotate_Selected
        return
    }

# check we have a valid default
    if {$defaulthtml == ""} {
        set msg "You must set a default html file for the map. \
            Please enter the name of a file (*.html) here"
        set defaulthtml [getEntry $msg]
    }

# if no file given, request one
    if {$mfile == ""} {
        set msg "Please give name of file to save map to (*.map). \ 
            (We shall put in a file browser here very shortly)."
        set mapfile [getEntry $msg]
    } else {
        set mapfile $mfile
    }

    set mapfile [addFileExtension $mapfile map]
    if {$mapfile == ""} {
        return
    }

# can the file be opened?
    if {[catch "set f [open $mapfile w]" msg] != 0} {
        postMessage "Cannot open mapfile ($mapfile) because: $msg"
        return
    }
    
# write the image file name if it exists
    if {![info exists imagefile] || $imagefile == ""} {
        postMessage "No image file specified, so no name written"
    } else {
        puts $f "# _IMAGEFILE $imagefile"
    }

# A default filename must be written to this file
    puts $f "default $defaulthtml"

    for {set i 0} {$i < $_NITEM} {incr i} {

        set itemIndex item$i
	set Tags [.can.c1 gettags $itemIndex]
        if {$Tags == ""} {
            continue
        }

	set Attributes [lindex $Tags 1]
	set Type [lindex $Attributes 0]
	set Colour [lindex $Attributes 1]
	set LineWidth [lindex $Attributes 2]
	set Comments [lindex $Attributes 3]
	set URL [lindex $Attributes 4]

# The imagemap files do not support "ovals".  Tcl calls both circles and ovals
# as "ovals".  If the type of the item is oval, it must be changed to circle.
	if {$Type=="oval"} {
	    set Coords [.can.c1 coords $itemIndex]
            set Type circle
# Tcl cirle coordinates contain one for the top left corner and one for the
# bottom right corner of a box surrounding the circle.  The imagemap files
# require the centre-point and a point on the edge. Therefore amend the 
# coordinates.
            set Centrex [expr (([lindex $Coords 2]+[lindex $Coords 0])/2)]
            set Centrey [expr (([lindex $Coords 3]+[lindex $Coords 1])/2)]
            set Coords "$Centrex $Centrey $Centrex [lindex $Coords 3]"
        } else {
            set Coords [.can.c1 coords $itemIndex]
            if {$Type == "line" || $Type == "polygon"} {
                set Type poly
            }
            if {$Type == "rectangle"} {
                set Type rect
            }
        }
# insert commas into coordinates
        set ncoord 0
        set CoordStg ""
        foreach coord $Coords {
            set CoordStg "$CoordStg$coord"
            if {$ncoord%2 == 0} {
                set CoordStg "$CoordStg,"
            } else {
                set CoordStg "$CoordStg "
            }
            incr ncoord
        }
     
        puts $f "# _COLOUR $Colour _LINEWIDTH $LineWidth "
        puts $f "#$Comments"
        puts $f "$Type $URL $CoordStg"
    }
    close $f
}

#=====================================================================#
#|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|/-\|#
#=====================================================================#

#---------------------------------------------------------------------#
# saves the map to the current map file                               #
#---------------------------------------------------------------------#
proc getDefaultHTML {} {

    global defaulthtml

    set defaulthtml [getEntry "Please enter the name of the HTML file to be \
        used for the default option"]

}

