#____________________________________________________________________________#
#|                                                                          |#
#| This file contains procedures to invoke the bindings allowing the user   |#
#| to create fixed shapes.  These include Rectangles and Circles.           |#
#| Ovals have been removed at present since the *.map file does not support |#
#| them.                                                                    |#
#| NB// There is a difference in the way coordinates for a  'radius defined |#
#|      circle' are calculated.                                             |#
#| 1. proc Create_Shape_BP1 :                                               |#
#|         Create the binding for setting the start coordinates of the      |#
#|         shape.                                                           |#
#| 2. proc Create_Shape_BM1 :                                               |#
#|         Binding to draw a bounding box that the shape will fit into.     |#
#| 3. proc Create_Shape_BR1 :                                               |#
#|         Create the specified shape at the calculated coordinates.        |#
#|__________________________________________________________________________|#
#                                                                            #

#============================================================================#

proc Create_Shape_BP1 {} {

# "Create_motion" is used to implement other bindings for this action
    uplevel #0 "set Create_motion 0"

    bind .can.c1 <ButtonPress-1> {
# Set var to hold coordinates for new item, and set "Create_motion" to allow 
# other binds to be activated
	set coords ""
	set Create_motion 1

# If a circle is to be drawn with the radius defining its size,
# a different bounding-box calculation must be executed
	if {$_CHOICE=="CIRCLE_RAD"} {
	    set centre_x [.can.c1 canvasx %x]
	    set centre_y [.can.c1 canvasy %y]
	} else {
# If any other shape drawing is selected bounding-box calculations
# are the identical.  Hold the coordinates for the corner in a list
	    set coords [concat $coords \
                [.can.c1 canvasx %x] [.can.c1 canvasy %y]]
	}
    }
}

#============================================================================#

proc Create_Shape_B1M {} {

    bind .can.c1 <B1-Motion> {
# Remove any previously drawn bounding-box
        .can.c1 delete box

# If the circle is to be defined by radius, draw the box with its centre the
# same as the centre of the circle.  Else draw the box starting from the point
# on the canvas that <BP-1> occured
        if {$_CHOICE=="CIRCLE_RAD"} {
            set dx [expr ([.can.c1 canvasx %x]-$centre_x)]

            set x1 [expr ($centre_x-$dx)]
            set y1 [expr ($centre_y-$dx)]
            set x2 [expr ($centre_x+$dx)]
            set y2 [expr ($centre_y+$dx)]

            eval ".can.c1 create rectangle $x1 $y1 $x2 $y2 -outline $_COLOUR \
                -width 1 -tags box"
        } else {
# The size is defined by change in x-direction for "circle_bb"
            if {$_CHOICE=="CIRCLE_BB"} {
                set dx [expr ([.can.c1 canvasx %x]-[lindex $coords 0])]

                set diffx [expr [lindex $coords 0]+$dx]
                set diffy [expr [lindex $coords 1]+$dx]

                eval ".can.c1 create rectangle $coords $diffx $diffy \
                    -outline $_COLOUR -width 1 -tags box"
            } else {
# For the "rectangle" and "oval" options, the box is defined by current
# x,y coordinates
                eval ".can.c1 create rectangle $coords [.can.c1 canvasx %x] \
                    [.can.c1 canvasy %y] -outline $_COLOUR -width 1 -tags box"
            }
        }
    }
}

#============================================================================#

proc Create_Shape_BR1 {} {

    bind .can.c1 <ButtonRelease-1> {
        # Remove the bounding-box from the canvas
        .can.c1 delete box

        # Set the list to hold the correct coordinates depending on which
        # option chosen
        if {$_CHOICE=="CIRCLE_RAD"} {
            set coords [list $x1 $y1 $x2 $y2]
        }
        if {$_CHOICE=="CIRCLE_BB"} {
            set coords [concat $coords $diffx $diffy]
        }
        if {$_CHOICE=="OVAL" || $_CHOICE=="RECTANGLE"} {
            set coords [concat $coords [.can.c1 canvasx %x] \
                [.can.c1 canvasy %y]]
        }

# For "circle_rad", "circle_bb" or "oval" the syntax is the same,
# with "$coords" defining the shape and size
        if {$_CHOICE=="CIRCLE_RAD" ||
            $_CHOICE=="CIRCLE_BB"  ||
            $_CHOICE=="OVAL"} {
            if {$_STYLE=="outline"} {
                eval ".can.c1 create oval $coords -outline $_COLOUR -fill {}\
                    -width $_LINE_WIDTH -tags item$_NITEM"
            } else {
                eval ".can.c1 create oval $coords -fill $_COLOUR -outline {} \
                    -tags item$_NITEM"
            }		    
        } else {
            if {$_STYLE=="outline"} {
                eval ".can.c1 create rectangle $coords -fill {} \
                    -outline $_COLOUR -width $_LINE_WIDTH -tags item$_NITEM"
            } else {
                eval ".can.c1 create rectangle $coords -fill $_COLOUR \
                    -outline {} -tags item$_NITEM"
            }
        }

# Call procedure to create a list of attributes for newly created items
        Item_Tags list item$_NITEM
# Add this list as the second tag
        .can.c1 addtag $list withtag item$_NITEM

# Give the tag "item" to the shape previously selected, in order that it
# can be reselected
        .can.c1 addtag item withtag selected

# Call procedure to automatically select the item last drawn
        Auto_Select item$_NITEM

# If a shape has been created reset the 'motion' variable
        set Create_motion 0

# Reset the radiobutton variable to empty
        set _CHOICE ""

# Call "proc Control" to reset the bindings to default
        Control ""
    }
}