################################################################################
# Create the bindings to allow user to scale the selected item.                #
# This procedure is called when no radiobutton (with var _CHOICE) is selected. #
################################################################################
proc Scale_Item {} {

# Set vars used to know which binding to implement depending on users actions
    uplevel #0 "set Scaling 0"
    uplevel #0 "set Move_motion 0"
    uplevel #0 "set Selecting 0"

# The following <Enter> and <Leave> binds allow the user to deselect all
# items in canvas
    .can.c1 bind item <Enter> {set Selecting 1}
    .can.c1 bind item <Leave> {set Selecting 0}
    .can.c1 bind selected <Leave> {set Selecting 0}


# Set the more general vars on <BP-1> in canvas

# A general <BP-1> in the canvas (not on a canvas item) deselects the selected
# shape.
    bind .can.c1 <ButtonPress-1> {
        if {$Scaling==0 && $Move_motion==0 && $Selecting==0} {
            .can.c1 delete boundingbox selpt copt
            .can.c1 addtag item withtag selected
            .can.c1 dtag selected
# disable Test_URL
            .can.menu.test configure -state disabled
        }
        set tagList [.can.c1 gettags current]

        set current_x [.can.c1 canvasx %x]
        set current_y [.can.c1 canvasy %y]
        set start_x $current_x
        set start_y $current_y
    }

# Add the binding for "Scaling" if <BP-1> in a point on the bounding-box
    .can.c1 bind selpt <ButtonPress-1> {+
# Set var to allow invoke of other bindings for this
        set Scaling 1
# Identify item type
        set type [.can.c1 type selected]

# Give the tag "scalept" to closest point to cursor
        .can.c1 addtag scalept closest \
            [.can.c1 canvasx %x] [.can.c1 canvasy %y] 1 "withtag selpt"
# Identify which point has been selected to use for scaling
        set thisPoint [lindex [.can.c1 gettags scalept] 0]

# Get a list of the coordinates o fa box around the item
        set coords [.can.c1 bbox selected]

# Set VARS to hold current canvas x,y coordinates
        set x1 [.can.c1 canvasx %x]
        set y1 [.can.c1 canvasy %y]

# The following if statements set the anchor position for the
# scale-function.  The anchor position is dependant on which point has
# been selected to scale with. The points are indexed by number in the
# following order around the bounding box :	1 2 3
#						4   5
#						6 7 8
        if {$thisPoint=="selpt1"} {
            set x0 [lindex $coords 2]; set y0 [lindex $coords 3]
        }

        if {$thisPoint=="selpt2"} {
            set x0 [lindex $coords 2]; set y0 [lindex $coords 3]
            set x1 [lindex $coords 0]
        }

        if {$thisPoint=="selpt3"} {
            set x0 [lindex $coords 0]; set y0 [lindex $coords 3]
        }

        if {$thisPoint=="selpt4"} {
            set x0 [lindex $coords 2]; set y0 [lindex $coords 3]
            set y1 [lindex $coords 1]
        }

        if {$thisPoint=="selpt5"} {
            set x0 [lindex $coords 0]; set y0 [lindex $coords 1]
            set y1 [lindex $coords 3]
        }

        if {$thisPoint=="selpt6"} {
            set x0 [lindex $coords 2]; set y0 [lindex $coords 1]
        }

        if {$thisPoint=="selpt7"} {
            set x0 [lindex $coords 0]; set y0 [lindex $coords 1]
            set x1 [lindex $coords 2]
        }

        if {$thisPoint=="selpt8"} {
            set x0 [lindex $coords 0]; set y0 [lindex $coords 1]
        }

    }
#----------------------------------------------------------------------------#
# <B1M> shows the user a box that the rescaled item will fill.  It is        #
# anchored around the same position as the scaling will occur from.          #
# If the item type is oval, extra code has been inserted to preserve the     #
# aspect-ratio.  Since the "*.map" file only supports circles.               #
# If the item is a circle then it can be scaled from any of points 2,4,5 or 7#
# and depending on the point selected, movement in the x/y planes are        #
# mirrored in the other plane.                                               #
#----------------------------------------------------------------------------#
    bind .can.c1 <B1-Motion> {
        .can.c1 delete sbox

        if {$Scaling} {
# If either of these points are selected and the item is not a circle
# scaling only need occur in the x-direction.
            if {$thisPoint=="selpt2" || $thisPoint=="selpt7"} {
                if {$type=="oval"} {
                    set tx [expr ($x0+([.can.c1 canvasy %y]-$y0))]
                    eval ".can.c1 create rectangle $x0 $y0 $tx \
                        [.can.c1 canvasy %y] -outline $_COLOUR -width 1 \
                            -tags sbox"
                } else {
                    eval ".can.c1 create rectangle $x0 $y0 $x1 \
                        [.can.c1 canvasy %y] -outline $_COLOUR -width 1 \
                        -tags sbox"
                }
            } else {
                if {$thisPoint=="selpt4" || $thisPoint=="selpt5"} {
                    if {$type=="oval"} {
                        set ty [expr ($y0+([.can.c1 canvasx %x]-$x0))]
                        eval ".can.c1 create rectangle $x0 $y0 \
                            [.can.c1 canvasx %x] $ty -outline $_COLOUR \
                            -width 1 -tags sbox"
                    } else {
                        eval ".can.c1 create rectangle $x0 $y0 \
                            [.can.c1 canvasx %x] $y1 -outline $_COLOUR \
                            -width 1 -tags sbox"
                    }
                } else {
                    if {$type=="oval"} {} else {
                        eval ".can.c1 create rectangle $x0 $y0 \
                            [.can.c1 canvasx %x] [.can.c1 canvasy %y] \
                            -outline $_COLOUR -width 1 -tags sbox"
                    }
                }
            }
        }
    }
#----------------------------------------------------------------------------#
# If the correct sequence of events has been followed the binding for <BR1>  #
# will calculate the amount to scale the item by.  Unfortunately tcl rounds  #
# decimal values to integers, and the scale command needs a decimal. See     #
# highlighted comments below for "A Way Around It"                           #
#----------------------------------------------------------------------------#
    bind .can.c1 <ButtonRelease-1> {
        .can.c1 delete sbox box
        if {$Scaling} {
# Reset the scaling variable 
            set Scaling 0
# Remove the visual aid for scaling from the canvas
            .can.c1 delete boundingbox

    #/-------------------------------------------------------------------\#
    #|###################################################################|#
    #|# A Way Around It :                                               #|#
    #|# Tcl does not support floating-point numbers, so by taking a     #|#
    #|# percentage value as a string and splitting to effectively give  #|#
    #|# a scaling factor to two decimal places, this can be overcome.   #|#
    #|# Full commenting is given for the x-plane.  The format is the    #|#
    #|# same for the y-plane, so refer to x-plane comments.             #|#
    #|###################################################################|#
    #\-------------------------------------------------------------------/#

# Set 'xScale' to the percentage value (NB:- *100) of the
# new x-value divided by the old x-value.
            set xScale [expr \
                ([expr ([.can.c1 canvasx %x]-$x0)*100]/[expr ($x1-$x0)])]

# Determine the length of the string containing this value.
# Note the '-1' is used becuase all indexes to the string
# start at zero, but count starrts at one.
            set xScaleLength [expr ([string length $xScale]-1)]

# Set variable to hold the last two elements of the string.
# This is the decimal part of the scaling factor.
            set xPoint [string range $xScale \
                [expr ($xScaleLength-1)] $xScaleLength]

# If the string is more than 2 elements long, set a variable
# to hold all previous characters in the correct order.
            if {[expr ($xScaleLength-1)]!=0} {
                set xInt [string range $xScale 0 [expr ($xScaleLength-2)]]

                set xInt1 [string range $xInt 0 0]
                if {$xInt1=="-"} {set xInt 1; set xPoint 0}
            } else {set xInt 0}

# 'Stringx' holds the decimal number to be used to scale in 
# the x-plane.
            set stringx "$xInt.$xPoint"

# The scaling factor in the y-plane is calculated in exactly the
# same way as for the x-plane.  For comments refer to above
            set yScale [expr \
                ([expr ([.can.c1 canvasy %y]-$y0)*100]/[expr ($y1-$y0)])]
            set yScaleLength [expr ([string length $yScale]-1)]
            set yPoint [string range $yScale \
                [expr ($yScaleLength-1)] $yScaleLength]

            if {[expr ($yScaleLength-1)]!=0} {
                set yInt [string range $yScale 0 [expr ($yScaleLength-2)]]
                set yInt1 [string range $yInt 0 0]
                if {$yInt1=="-"} {set yInt 1; set yPoint 0}
            } else {set yInt 0}

            set stringy "$yInt.$yPoint"

# If scaling is only required in one plane, the following if
# statements catch this and amend the relevant scaling-factor value.
            if {$thisPoint=="selpt2" || $thisPoint=="selpt7"} {set stringx 1.0}
            if {$thisPoint=="selpt4" || $thisPoint=="selpt5"} {set stringy 1.0}

# Once the scaling event has occurred remove the tag 
# 'scalept' from the item with it.
            .can.c1 dtag scalept

# If the item is a circle, the following nested if statements ensure
# that the aspect-ratio is preserved.
# The final part sets the scaling-factor to 1.0 if a corner is
# selected to scale with
            if {$type=="oval"} {
                if {$thisPoint=="selpt2" || $thisPoint=="selpt7"} {
                    set stringx $stringy
                } else {
                    if {$thisPoint=="selpt4" || $thisPoint=="selpt5"} {
                        set stringy $stringx
                    } else {
                        set stringx "1.0"
                        set stringy "1.0"
                    }
                }
            }

# This is the scale command. See manual entry for syntax.
            eval ".can.c1 scale selected $x0 $y0 $stringx $stringy"

# Identify which item has been scaled and reselect it
            set itemindex [lindex [.can.c1 gettags selected] 0]

            Auto_Select $itemindex
        }
    }
}